package FPGrowthRuleMining;

import java.util.Hashtable;
import java.util.List;

public class FPTree {
	//Variables
	private FPTreeNode root;
	private FPTreeNode[] nodeLinkTable;
	private Hashtable<String, Integer> freqPatternTable;
	private int minSupport;
	
	//Constructor
	public FPTree(int nodeLinkSize, Hashtable<String, Integer> freqPatternTable, int support) {
		this.root = new FPTreeNode();
		this.nodeLinkTable = new FPTreeNode[nodeLinkSize];
		this.freqPatternTable = freqPatternTable;
		this.minSupport = support;
	}
	
	//Methods
	public FPTreeNode getRoot() {
		return this.root;
	}
	
	public FPTreeNode[] getTable() {
		return this.nodeLinkTable;
	}
	
	public void constructTree(List<Itemset> condPattBase) {
		FPTreeNode current = this.root;
		for(Itemset itemset : condPattBase) {
			current = this.root;
			for(String item : itemset.getItemset()) {
				boolean childExisted = current.hasChild(item) != (-1);
				int tableIndex = freqPatternTable.get(item);
				FPTreeNode child = current.addChild(item, itemset.getSupport());
				
				if(nodeLinkTable[tableIndex] == null) {
					nodeLinkTable[tableIndex] = child;
				}
				else if(!childExisted) {
					FPTreeNode next = nodeLinkTable[tableIndex];
					while(next.hasNext()) {
						next = next.getNext();
					}
					next.setNext(child);
				}
				current = child;
			}
		}
	}
	
	public void removeInfrequent() {
		for(int i=(this.nodeLinkTable.length-1); i>=0; i--) {
			if(nodeLinkTable[i] == null) {
				continue;
			}
			
			//count support
			FPTreeNode current = nodeLinkTable[i];
			int support = current.getCount();
			while(current.hasNext()) {
				current = current.getNext();
				support += current.getCount();
			}
			
			//remove them if infrequent
			current = nodeLinkTable[i];
			if(support < this.minSupport) {
				while(current.hasNext()) {
					removeNode(current.getNext(), nodeLinkTable[i]);
				}
				nodeLinkTable[i] = null;
			}
		}
	}
	
	public void removeNode(FPTreeNode node, FPTreeNode prev) {
		FPTreeNode parent = node.getParent();
		parent.removeChild(node.getItem());//node parent link and parent's child link is deleted
		String[] children = new String[node.getChildrenNodes().size()];
		int counter= 0;
		for(FPTreeNode n : node.getChildrenNodes()) {
			parent.addChild(n.getItem(), n.getCount());
			children[counter] = n.getItem();
			counter++;
		}
		for(int i=0; i<children.length; i++) {
			node.removeChild(children[i]);
		}
		
		prev.setNext(node.getNext());
		node.setNext(null);
	}
}
